program chmbc2cdf

  use netcdf
  implicit none

#define NCERR(lnum) if(ncstat/=NF90_NOERR) call nc_err(ncstat,lnum,'outprint.f90')

  ! user parameter
  character(len=*),parameter :: fniARGS = 'chmbc2cdf.nml'

  ! system parameters. Do not change
  integer,parameter :: dlen=19
  real(kind=8),parameter :: dzero= 0d0

  !  constant netCDF attributes
  character(len=*),parameter :: title=      'CHIMERE SUITE'
  character(len=*),parameter :: subtitle=   'Hourly output concentrations file'
  character(len=*),parameter :: generator=  'Generated by chmbin2cdf'
  character(len=*),parameter :: conventions='None'


  ! strings
  character(len=dlen) :: datebuf
  character(len=1024) :: history
  character(len=15)   :: long_name

  character(len=132) :: fnout         ! Hourly concentrations, netCDF
  character(len=132) :: fnin          ! Hourly concentrations, binary
  character(len=132) :: fnoutspec  ! Ordered list of species
  character(len=132) :: fcoord       ! Coordinates file

  character(len=16)  :: domain

  ! scalars
  integer :: nzonal
  integer :: nmerid
  integer :: nivout
  integer :: nspectot
  integer :: nphour
  integer :: nprint

  integer :: izo,ime,ilev

  integer :: ifn_args,ierr
  integer :: iospec
  integer :: ncstat
  integer :: out_ncid

  integer :: out_time_dimid
  integer :: out_date_dimid
  integer :: out_zonal_dimid
  integer :: out_merid_dimid
  integer :: out_layers_dimid
!
  integer :: out_verti_dimid
  integer :: out_bins_dimid
  integer::ms,nverti,idummy
!
  integer :: out_lon_varid
  integer :: out_lat_varid
  integer :: out_times_varid
  integer :: out_hlay_varid
  integer :: out_airm_varid


  integer :: ifn_in
  integer :: ifn_coord
  integer :: ifn_outspec

  integer :: ispec,nd
  integer :: idate      ! chimere date as YYYYMMDDHH

  real(kind=8) :: xmass


! modifiche per conpatibilita'

  character(len=132) :: fvcoord      ! File containing the vertical hybrid coefficients
  integer::ilay,nlayers,l

 real,allocatable,dimension(:) :: avcoord !
  real,allocatable,dimension(:) :: bvcoord ! coefficients for layers (top) in CHIMERE 
  integer::out_avcoord_varid,out_bvcoord_varid,out_cutoff_varid
  real(kind=8),allocatable::ds(:)  ! sectionnnal diameter

  integer::ifnaerosol
  character(len=132)::fnaerosol           !
  integer :: ifn_vcoo
!fine modifiche



    

  ! arrays

  real,allocatable,dimension(:,:,:,:) :: rbuf4,toprint
  real,allocatable,dimension(:,:,:)   :: hlaybuf,airmbuf
  real,allocatable,dimension(:,:)     :: lon,lat

  ! System information stuff
  character(len=32) :: systime
  character(len=64) :: hname
  character(len=32) :: usrname
  character(len=255) :: cwd

  ! Chimere types
  ! A structure to hold informations about the fields to write to file "out"
  type :: output_species_type
     real(kind=8)      :: fspec          ! conversion factor
     integer           :: varid          ! netcdf var id
     !integer           :: iaddr          ! address in species(:) table
     character(len=16) :: name           ! name from species(:) table
     character(len=16) :: units
  end type output_species_type
  type(output_species_type),allocatable,dimension(:) :: output_species

  ! Functions
  character(len=dlen) :: numeric2mm5date
  !*****************************************************************************************

  namelist /args/    &
       fnout,        &
       fnin,         &
       fnoutspec, &
       fcoord,      &
       fvcoord,      &
       fnaerosol,     &
       nzonal,       &
       nmerid,       &
       nivout,       &
       nspectot,     &
       domain,       &
       nphour,       &
       nprint


  !*****************************************************************************************
 write(6,*)'aaaaaaaa'
  ! read arguments namelist
  call opfi(ifn_args,fniARGS,'f','o')
  read(ifn_args,nml=args,iostat=ierr)
  write(6,*)     fnout
  write(6,*)     fnin
  write(6,*)     fnoutspec
  write(6,*)      fcoord
 write(6,*)      fvcoord
  write(6,*)     fnaerosol
  write(6,*)     nzonal
  write(6,*)     nmerid
  write(6,*)     nivout
  write(6,*)     nspectot
 write(6,*)      domain
 write(6,*)      nphour
  write(6,*)     nprint
  if (ierr/=0) then
     print *,'iniread: problem reading arguments namelist'
     stop
  end if

  ! some initializations
  allocate(rbuf4(nzonal,nmerid,nivout,nspectot))
  allocate(toprint(nspectot,nzonal,nmerid,nivout))
  allocate(hlaybuf(nzonal,nmerid,nivout))
  allocate(airmbuf(nzonal,nmerid,nivout))
  allocate(output_species(nspectot))
  do ispec=1,nspectot
     output_species(ispec)%name  = '                '
     output_species(ispec)%units = '                '
  end do

! open aersol file 
  call opfi(ifnaerosol,fnaerosol,'f','o')
  read(ifnaerosol,*)
  read(ifnaerosol,*)ms,idummy,idummy
  write(6,*)ms,idummy,idummy
  allocate(ds(ms+1))
  read(ifnaerosol,*) (ds(l),l=1,ms+1)
  
 
  nverti=8
  nlayers=8
  allocate(avcoord(nlayers))
  allocate(bvcoord(nlayers))

  ! open binary chimere file
  call opfi(ifn_in,fnin,'u','o')


  ! read coarse grid coordinates
  allocate(lon(nzonal,nmerid))
  allocate(lat(nzonal,nmerid))
  call opfi(ifn_coord,fcoord,'f','o')
  do ime=1,nmerid
     do izo=1,nzonal
        read(ifn_coord,*) lon(izo,ime),lat (izo,ime)
     enddo
  enddo
  close(ifn_coord)

  ! read output species list
  call opfi(ifn_outspec,fnoutspec,'f','o')
  do ispec=1,nspectot
     read(ifn_outspec,*,end=1005) output_species(ispec)%name, xmass
     output_species(ispec)%fspec = xmass*1.6603e-12
  enddo
1005 continue
  close(ifn_outspec)

  ! get various system informations
  call get_system(usrname,hname,systime,cwd)

  ! create out file
  ncstat=nf90_create(fnout,NF90_CLOBBER,out_ncid)
  NCERR(__LINE__)

  ! create dimensions in out file
  call create_dims_out

  ! create variables in out file
  call create_vars_out

  ! write attributes to out file
  call create_atts_out

  ! end define mode
  ncstat=nf90_enddef(out_ncid)
  NCERR(__LINE__)




 ! open vcoord chimere file
  call opfi(ifn_vcoo,fvcoord,'f','o')

  ! Reading vertical hybrid coefficients
  do ilay=1,nlayers
     read(ifn_vcoo,*,iostat=ierr) avcoord(ilay),bvcoord(ilay)
     write(6,*)avcoord(ilay),bvcoord(ilay)
     if (ierr/=0) stop '*** diagmet: error reading Vertical Hybrid Coefficients file'
  enddo


!




  ! record longitude and latitude
  ncstat=nf90_put_var(out_ncid,out_lon_varid,lon)
  NCERR(__LINE__)
  ncstat=nf90_put_var(out_ncid,out_lat_varid,lat)
  NCERR(__LINE__)

  ! record vcoord coefficient
  ncstat=nf90_put_var(out_ncid,out_avcoord_varid,avcoord)
  NCERR(__LINE__)
  ncstat=nf90_put_var(out_ncid,out_bvcoord_varid,bvcoord)
  NCERR(__LINE__)
  ! record bin cut off diameters coefficient
  ncstat=nf90_put_var(out_ncid,out_cutoff_varid,ds)
  NCERR(__LINE__)





  do nd=1,10000000
     read(ifn_in,end=1001)idate,((((rbuf4(izo,ime,ilev,ispec),izo=1,nzonal)     &
          ,ime=1,nmerid),ilev=1,nivout),ispec=1,nspectot)                  & 
          ,(((hlaybuf(izo,ime,ilev),izo=1,nzonal),ime=1,nmerid),ilev=1,nivout) &
          ,(((airmbuf(izo,ime,ilev),izo=1,nzonal),ime=1,nmerid),ilev=1,nivout)
     write(6,*)'datain' ,idate,nspectot
     if(idate<20080521 .and. ms>6) then  
     write(6,*) 'dati non  disponibili' 
     exit
     end if
     if(idate>20080521 ) then  
          do ispec=1,nspectot
           write(6,'(i3,a10,2f10.5)')ispec,output_species(ispec)%name,&
      sum(rbuf4(1:nzonal,1:nmerid,1:nivout,ispec))/(nzonal*nmerid*nivout),&
           rbuf4(1,1,1,ispec)
          enddo
          write(6,*)hlaybuf(1,1,1),hlaybuf(nzonal,nmerid,1),airmbuf(1,1,1)
     end if
     do ilev=1,nivout
        do ime=1,nmerid
           do izo=1,nzonal
              do ispec=1,nspectot
                 toprint(ispec,izo,ime,ilev)=rbuf4(izo,ime,ilev,ispec)
!                 toprint(ispec,izo,ime,ilev)=0
              end do
              if(ilev==1) then
!              write(77,*)hlaybuf(izo,ime,1) 
              end if
           end do
        end do
     end do

     ! TIMES
     datebuf=numeric2mm5date(idate)
     print *,datebuf
     write(unit= datebuf(15:16),fmt='(i2.2)')  mod(60*nprint*(nd-1)/nphour,60)
     ncstat=nf90_put_var(out_ncid,out_times_varid,datebuf,(/1,nd/),(/dlen,1/))
     NCERR(__LINE__)

     ! SPECIES
     do ispec=1,nspectot
        ncstat=nf90_put_var(                              &
             out_ncid,                                    &
             output_species(ispec)%varid,                 &
             toprint(ispec,:,:,:),                        &
             (/1,1,1,nd/),(/nzonal,nmerid,nivout,1/)      &
             )
        NCERR(__LINE__)
     end do

     ! HLAY
     ncstat=nf90_put_var(                                 &
          out_ncid,                                       &
          out_hlay_varid,                                 &
          hlaybuf,                                        &
          (/1,1,1,nd/),(/nzonal,nmerid,nivout,1/)         &
          )
     NCERR(__LINE__)
     ! AIRM
     ncstat=nf90_put_var(                                 &
          out_ncid,                                       &
          out_airm_varid,                                 &
          airmbuf,                                        &
          (/1,1,1,nd/),(/nzonal,nmerid,nivout,1/)         &
          )
     NCERR(__LINE__)

  end do
1001 continue





  ncstat=nf90_close(out_ncid)
  NCERR(__LINE__)



contains

  !****************************************************
  subroutine create_dims_out
    implicit none

    ncstat=nf90_def_dim(out_ncid,'Time',        NF90_UNLIMITED,out_time_dimid)
    NCERR(__LINE__)
    ncstat=nf90_def_dim(out_ncid,'DateStrLen',  dlen,          out_date_dimid)
    NCERR(__LINE__)
    ncstat=nf90_def_dim(out_ncid,'west_east',   nzonal,        out_zonal_dimid)
    NCERR(__LINE__)
    ncstat=nf90_def_dim(out_ncid,'south_north', nmerid,        out_merid_dimid)
    NCERR(__LINE__)
    ncstat=nf90_def_dim(out_ncid,'bottom_top',  nivout,       out_layers_dimid)
    NCERR(__LINE__)
    ncstat=nf90_def_dim(out_ncid,'vcoord_dim',  nverti,       out_verti_dimid)
    NCERR(__LINE__)
    ncstat=nf90_def_dim(out_ncid,'number_of_cut_off_diameters',  ms+1,       out_bins_dimid)
    NCERR(__LINE__)
  end subroutine create_dims_out

  !****************************************************
  subroutine create_atts_out
    implicit none

    ncstat=nf90_put_att(out_ncid,NF90_GLOBAL,'Title',title)
    NCERR(__LINE__)
    ncstat=nf90_put_att(out_ncid,NF90_GLOBAL,'Sub-title',subtitle)
    NCERR(__LINE__)
    ncstat=nf90_put_att(out_ncid,NF90_GLOBAL,'Chimere_type','out')
    NCERR(__LINE__)
    ncstat=nf90_put_att(out_ncid,NF90_GLOBAL,'Generating_process',generator)
    NCERR(__LINE__)
    ncstat=nf90_put_att(out_ncid,NF90_GLOBAL,'Conventions',conventions)
    NCERR(__LINE__)
    ncstat=nf90_put_att(out_ncid,NF90_GLOBAL,'Domain',domain)
    NCERR(__LINE__)

#if defined(IFORT)

    history = &
         'File '//fnout(1:len_trim(fnout))      // &
         ' was generated on '                   // &
         systime(1:len_trim(systime))           // &
         ' by '                                 // &
         usrname(1:len_trim(usrname))           // &
         ' on '                                 // &
         hname(1:len_trim(hname))               //'\n'C
    history = history(1:len_trim(history)-1)

#elif (defined(G95)+defined(PGI))

    history =                                       &
         'File '//fnout(1:len_trim(fnout))       // &
         ' was generated on '                    // &
         systime(1:len_trim(systime))            // &
         ' by '                                  // &
         usrname(1:len_trim(usrname))            // &
         ' on '                                  // &
         hname(1:len_trim(hname))
    history = history(1:len_trim(history)-1)

#else

    history='unknown'

#endif

    ncstat=nf90_put_att(out_ncid,NF90_GLOBAL,'history',history(1:len_trim(history)))
    NCERR(__LINE__)

  end subroutine create_atts_out


  !****************************************************
  subroutine create_vars_out
    implicit none

    ! LON
    ncstat=nf90_def_var(                                                         &
         out_ncid,                                                               &
         'lon',                                                                  &
         NF90_FLOAT,                                                             &
         (/out_zonal_dimid,out_merid_dimid/),                                    &
         out_lon_varid                                                           &
         )
    NCERR(__LINE__)
    ncstat=nf90_put_att(                                                         &
         out_ncid,                                                               &
         out_lon_varid,                                                          &
         'units',                                                                &
         'degrees_east'                                                          &
         )
    NCERR(__LINE__)
    ncstat=nf90_put_att(                                                         &
         out_ncid,                                                               &
         out_lon_varid,                                                          &
         'long_name',                                                            &
         'Longitude'                                                             &
         )
    NCERR(__LINE__)

    ! LAT
    ncstat=nf90_def_var( &
         out_ncid, &
         'lat', &
         NF90_FLOAT, &
         (/out_zonal_dimid,out_merid_dimid/),                                    &
         out_lat_varid                                                           &
         )
    NCERR(__LINE__)
    ncstat=nf90_put_att(                                                         &
         out_ncid,                                                               &
         out_lat_varid,                                                          &
         'units',                                                                &
         'degrees_north'                                                         &
         )
    NCERR(__LINE__)
    ncstat=nf90_put_att(                                                         &
         out_ncid,                                                               &
         out_lat_varid,                                                          &
         'long_name',                                                            &
         'Latitude'                                                              &
         )
    NCERR(__LINE__)
! aggiunta per compatibilita



 ! AVCOORD
    ncstat=nf90_def_var( &
         out_ncid, &
         'a_vcoord', &
         NF90_FLOAT, &
         (/out_verti_dimid/),                                    &
         out_avcoord_varid                                                           &
         )
    NCERR(__LINE__)
    ncstat=nf90_put_att(                                                         &
         out_ncid,                                                               &
         out_avcoord_varid,                                                          &
         'units',                                                                &
         'no_units'                                                         &
         )
    NCERR(__LINE__)
    ncstat=nf90_put_att(                                                         &
         out_ncid,                                                               &
         out_avcoord_varid,                                                          &
         'long_name',                                                            &
         'A_sigma_coefficient'                                                              &
         )
    NCERR(__LINE__)

    ! BVCOORD
    ncstat=nf90_def_var( &
         out_ncid, &
         'b_vcoord', &
         NF90_FLOAT, &
         (/out_verti_dimid/),                                    &
         out_bvcoord_varid                                                           &
         )
    NCERR(__LINE__)
    ncstat=nf90_put_att(                                                         &
         out_ncid,                                                               &
         out_bvcoord_varid,                                                          &
         'units',                                                                &
         'no_units'                                                         &
         )
    NCERR(__LINE__)
    ncstat=nf90_put_att(                                                         &
         out_ncid,                                                               &
         out_bvcoord_varid,                                                          &
         'long_name',                                                            &
         'B_sigma_coefficient'                                                              &
         )
    NCERR(__LINE__)

    !CUT OFF DIAMETERS 
    ncstat=nf90_def_var( &
         out_ncid, &
         'cut_off_diameters', &
         NF90_FLOAT, &
         (/out_bins_dimid/),                                    &
         out_cutoff_varid                                                           &
         )
    NCERR(__LINE__)
    ncstat=nf90_put_att(                                                         &
         out_ncid,                                                               &
         out_cutoff_varid,                                                          &
         'units',                                                                &
         'meters'                                                         &
         )
    NCERR(__LINE__)
    ncstat=nf90_put_att(                                                         &
         out_ncid,                                                               &
         out_cutoff_varid,                                                          &
         'long_name',                                                            &
         'cut_off_diameters'                                                              &
         )
    NCERR(__LINE__)
! fine aggiunt




!
    ! TIMES
    ncstat=nf90_def_var(                                                         &
         out_ncid,                                                               &
         'Times',                                                                &
         NF90_CHAR,                                                              &
         (/out_date_dimid,out_time_dimid/),                                      &
         out_times_varid                                                         &
         )
    NCERR(__LINE__)

    ! OUT SPECIES
    do iospec=1,nspectot
       ! set units
       if(output_species(iospec)%fspec.gt.dzero) then
          output_species(iospec)%units='ug/m3'
       else
          output_species(iospec)%units='ppb vol'
       endif

       ! write values
       ncstat=nf90_def_var(                                                      &
            out_ncid,                                                            &
            output_species(iospec)%name,                                         &
            NF90_FLOAT,                                                          &
            (/out_zonal_dimid,out_merid_dimid,out_layers_dimid,out_time_dimid/), &
            output_species(iospec)%varid                                         &
            )
       NCERR(__LINE__)
       ! write attributes
       ncstat=nf90_put_att(                                                      &
            out_ncid,                                                            &
            output_species(iospec)%varid,                                        &
            'units',                                                             &
            output_species(iospec)%units                                         &
            )
       NCERR(__LINE__)
       long_name=output_species(iospec)%name
       ncstat=nf90_put_att(                                                      &
            out_ncid,                                                            &
            output_species(iospec)%varid,                                        &
            'long_name',                                                         &
            long_name(1:len_trim(long_name))//' Concentration'                   &
            )
       NCERR(__LINE__)
    end do

    ! HLAY
    ncstat=nf90_def_var(                                                         &
         out_ncid,                                                               &
         'hlay',                                                                 &
         NF90_FLOAT,                                                             &
         (/out_zonal_dimid,out_merid_dimid,out_layers_dimid,out_time_dimid/),    &
         out_hlay_varid                                                          &
         )
    NCERR(__LINE__)
    ncstat=nf90_put_att(                                                         &
         out_ncid,                                                               &
         out_hlay_varid,                                                         &
         'units',                                                                &
         'meter'                                                                 &
         )
    NCERR(__LINE__)
    ncstat=nf90_put_att(                                                         &
         out_ncid,                                                               &
         out_hlay_varid,                                                         &
         'long_name',                                                            &
         'Layer top altitude'                                                    &
         )
    NCERR(__LINE__)

    ! AIRM
    ncstat=nf90_def_var(                                                         &
         out_ncid,                                                               &
         'airm',                                                                 &
         NF90_FLOAT,                                                             &
         (/out_zonal_dimid,out_merid_dimid,out_layers_dimid,out_time_dimid/),    &
         out_airm_varid                                                          &
         )
    NCERR(__LINE__)
    ncstat=nf90_put_att(                                                         &
         out_ncid,                                                               &
         out_airm_varid,                                                         &
         'units',                                                                &
         'molecule/cm**3'                                                        &
         )
    NCERR(__LINE__)
    ncstat=nf90_put_att(                                                         &
         out_ncid,                                                               &
         out_airm_varid,                                                         &
         'long_name',                                                            &
         'Air density'                                                           &
         )
    NCERR(__LINE__)

  end subroutine create_vars_out





end program chmbc2cdf
